@@ -31,21 +31,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 // Local
 
 
-class QLabel;
 
 class QAction;
 
 namespace Gwenview {
 
 
-class ZoomWidgetPrivate;
+struct ZoomWidgetPrivate;
 class GWENVIEWLIB_EXPORT ZoomWidget : public QFrame {
 	Q_OBJECT
 public:
 	ZoomWidget(QWidget* parent = 0);
 	~ZoomWidget();
 
-	void setActions(QAction* zoomToFitAction, QAction* actualSizeAction);
+	void setActions(QAction* zoomToFitAction, QAction* actualSizeAction, QAction* zoomInAction, QAction* zoomOutAction);
 
 	void setZoom(qreal zoom);
 
@@ -59,7 +58,7 @@ private Q_SLOTS:
 	void slotZoomSliderActionTriggered();
 
 private:
-	friend class ZoomWidgetPrivate;
+	friend struct ZoomWidgetPrivate;
 	ZoomWidgetPrivate* const d;
 };
 
