@@ -23,6 +23,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // Qt
 #include <QObject>
 
+// KDE
+#include <kjob.h>
+
 // Local
 #include "../lib/document/document.h"
 
@@ -48,6 +51,49 @@ public:
 };
 
 
+class JobWatcher : public QObject {
+	Q_OBJECT
+public:
+	JobWatcher(KJob* job)
+	: mJob(job)
+	, mDone(false)
+	, mError(0)
+	{
+		connect(job, SIGNAL(result(KJob*)),
+			SLOT(slotResult(KJob*)));
+		connect(job, SIGNAL(destroyed(QObject*)),
+			SLOT(slotDestroyed()));
+	}
+
+	void wait() {
+		while (!mDone) {
+			QApplication::processEvents();
+		}
+	}
+
+	int error() const {
+		return mError;
+	}
+
+private Q_SLOTS:
+	void slotResult(KJob* job) {
+		mError = job->error();
+		mDone = true;
+	}
+
+	void slotDestroyed() {
+		kWarning() << "Destroyed";
+		mError = -1;
+		mDone = true;
+	}
+
+private:
+	KJob* mJob;
+	bool mDone;
+	int mError;
+};
+
+
 class DocumentTest : public QObject {
 	Q_OBJECT
 
@@ -64,15 +110,18 @@ private Q_SLOTS:
 	void testDeleteWhileLoading();
 	void testLoadRotated();
 	void testMultipleLoads();
-	void testSave();
+	void testSaveAs();
 	void testSaveRemote();
 	void testLosslessSave();
 	void testLosslessRotate();
-	void testModify();
+	void testModifyAndSaveAs();
 	void testMetaInfoJpeg();
 	void testMetaInfoBmp();
 	void testForgetModifiedDocument();
 	void testModifiedAndSavedSignals();
+	void testJobQueue();
+	void testCheckDocumentEditor();
+	void testUndoStackPush();
 
 	void initTestCase();
 	void init();
