@@ -1,14 +1,16 @@
-/* ============================================================
+/** ===========================================================
+ * @file
  *
- * This file is a part of kipi-plugins project
- * http://www.kipi-plugins.org
+ * This file is a part of digiKam project
+ * <a href="http://www.digikam.org">http://www.digikam.org</a>
  *
- * Date        : 2008-08-16
- * Description : Integer and double num input widget
- *               re-implemented with a reset button to switch to
- *               a default value.
+ * @date   2008-08-16
+ * @brief  Integer and double num input widget
+ *         re-implemented with a reset button to switch to
+ *         a default value
  *
- * Copyright (C) 2008-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * @author Copyright (C) 2008-2010 by Gilles Caulier
+ *         <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -18,12 +20,11 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * ============================================================ */
 
-#include "rnuminput.h"
 #include "rnuminput.moc"
 
 // Qt includes.
@@ -39,7 +40,7 @@
 namespace KDcrawIface
 {
 
-class RIntNumInputPriv
+class RIntNumInput::RIntNumInputPriv
 {
 
 public:
@@ -53,12 +54,12 @@ public:
 
     int           defaultValue;
 
-    QToolButton  *resetButton;
+    QToolButton*  resetButton;
 
-    KIntNumInput *input;
+    KIntNumInput* input;
 };
 
-RIntNumInput::RIntNumInput(QWidget *parent)
+RIntNumInput::RIntNumInput(QWidget* parent)
             : KHBox(parent), d(new RIntNumInputPriv)
 {
     d->input       = new KIntNumInput(this);
@@ -132,15 +133,13 @@ void RIntNumInput::slotReset()
 
 void RIntNumInput::slotValueChanged(int v)
 {
-    if (v != d->defaultValue)
-        d->resetButton->setEnabled(true);
-
+    d->resetButton->setEnabled(v != d->defaultValue);
     emit valueChanged(v);
 }
 
 // ----------------------------------------------------
 
-class RDoubleNumInputPriv
+class RDoubleNumInput::RDoubleNumInputPriv
 {
 
 public:
@@ -159,7 +158,7 @@ public:
     KDoubleNumInput *input;
 };
 
-RDoubleNumInput::RDoubleNumInput(QWidget *parent)
+RDoubleNumInput::RDoubleNumInput(QWidget* parent)
                : KHBox(parent), d(new RDoubleNumInputPriv)
 {
     d->input       = new KDoubleNumInput(this);
@@ -233,9 +232,7 @@ void RDoubleNumInput::slotReset()
 
 void RDoubleNumInput::slotValueChanged(double v)
 {
-    if (v != d->defaultValue)
-        d->resetButton->setEnabled(true);
-
+    d->resetButton->setEnabled(v != d->defaultValue);
     emit valueChanged(v);
 }
 
