@@ -1,23 +1,26 @@
-/* ============================================================
+/** ===========================================================
+ * @file
  *
- * This file is a part of kipi-plugins project
- * http://www.kipi-plugins.org
+ * This file is a part of digiKam project
+ * <a href="http://www.digikam.org">http://www.digikam.org</a>
  *
- * Date        : 2006-09-15
- * Description : Exiv2 library interface for KDE
- *               Xmp manipulation methods
+ * @date   2006-09-15
+ * @brief  Xmp manipulation methods
  *
- * Copyright (C) 2006-2010 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2010 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * @author Copyright (C) 2006-2010 by Gilles Caulier
+ *         <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
+ * @author Copyright (C) 2006-2010 by Marcel Wiesweg
+ *         <a href="mailto:marcel dot wiesweg at gmx dot de">marcel dot wiesweg at gmx dot de</a>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
- * either version 2, or (at your option) any later version.
+ * either version 2, or (at your option)
+ * any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * ============================================================ */
@@ -44,12 +47,10 @@ bool KExiv2::canWriteXmp(const QString& filePath)
     catch( Exiv2::Error& e )
     {
         std::string s(e.what());
-        kDebug(51003) << "Cannot check Xmp access mode using Exiv2 (Error #"
-                      << e.code() << ": " << s.c_str() << ")" << endl;
+        kDebug() << "Cannot check Xmp access mode using Exiv2 (Error #"
+                      << e.code() << ": " << s.c_str() << ")";
     }
 
-    return false;
-
 #else
 
     Q_UNUSED(filePath);
@@ -139,7 +140,7 @@ bool KExiv2::setXmp(const QByteArray& data) const
     catch( Exiv2::Error& e )
     {
         if (!d->filePath.isEmpty())
-            kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
+            kDebug() << "From file " << d->filePath.toAscii().constData();
 
         d->printExiv2ExceptionError("Cannot set Xmp data using Exiv2 ", e);
     }
@@ -272,7 +273,6 @@ QString KExiv2::getXmpTagTitle(const char* xmpTagName)
 QString KExiv2::getXmpTagDescription(const char* xmpTagName)
 {
 #ifdef _XMP_SUPPORT_
-
     try
     {
         std::string xmpkey(xmpTagName);
@@ -342,8 +342,7 @@ bool KExiv2::setXmpTagString(const char* xmpTagName, const QString& value, bool
         const std::string &txt(value.toUtf8().constData());
         Exiv2::Value::AutoPtr xmpTxtVal = Exiv2::Value::create(Exiv2::xmpText);
         xmpTxtVal->read(txt);
-
-        d->xmpMetadata().add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
+        d->xmpMetadata()[xmpTagName].setValue(xmpTxtVal.get());
         return true;
     }
     catch( Exiv2::Error& e )
@@ -533,7 +532,7 @@ bool KExiv2::setXmpTagStringLangAlt(const char* xmpTagName, const QString& value
                 {
                     const std::string &val((*it).toUtf8().constData());
                     xmpTxtVal->read(val);
-                    kDebug(51003) << *it << endl;
+                    kDebug() << *it;
                 }
             }
         }
@@ -586,7 +585,7 @@ QStringList KExiv2::getXmpTagStringSeq(const char* xmpTagName, bool escapeCR) co
 
                     seq.append(seqValue);
                 }
-                kDebug(51003) << "XMP String Seq (" << xmpTagName << "): " << seq << endl;
+                kDebug() << "XMP String Seq (" << xmpTagName << "): " << seq;
 
                 return seq;
             }
