@@ -5,7 +5,8 @@
  * Date        : 2009-01-24
  * Description : Sane interface for KDE
  *
- * Copyright (C) 2007-2009 by Kare Sars <kare dot sars at iki dot fi>
+ * Copyright (C) 2007-2010 by Kare Sars <kare dot sars at iki dot fi>
+ * Copyright (C) 2009 by Matthias Nagl <matthias at nagl dot info>
  * Copyright (C) 2009 by Grzegorz Kurtyka <grzegorz dot kurtyka at gmail dot com>
  * Copyright (C) 2007-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
@@ -30,10 +31,15 @@
 #include "ksane.h"
 #include "ksane.moc"
 
+#include <unistd.h>
+
 // Qt includes
 #include <QApplication>
 #include <QVarLengthArray>
 #include <QLabel>
+#include <QSplitter>
+#include <QMutex>
+#include <QPointer>
 
 // KDE includes
 #include <kpassworddialog.h>
@@ -56,14 +62,8 @@
 
 namespace KSaneIface
 {
-    static QString sane_username;
-    static QString sane_password;
-
-/** static function called by sane_open to get authorization from user */
-static void getSaneAuthorization(SANE_String_Const, SANE_Char *username, SANE_Char *password) {
-    qstrncpy(username, sane_username.toUtf8(), SANE_MAX_USERNAME_LEN);
-    qstrncpy(password, sane_password.toUtf8(), SANE_MAX_PASSWORD_LEN );
-}
+static int     s_objectCount = 0;
+static QMutex  s_objectMutex;
 
 KSaneWidget::KSaneWidget(QWidget* parent)
     : QWidget(parent), d(new KSaneWidgetPrivate)
@@ -71,28 +71,40 @@ KSaneWidget::KSaneWidget(QWidget* parent)
     SANE_Int    version;
     SANE_Status status;
 
-    //kDebug(51004) <<  "The language is:" << KGlobal::locale()->language();
-    //kDebug(51004) <<  "Languagelist" << KGlobal::locale()->languageList();
+    //kDebug() <<  "The language is:" << KGlobal::locale()->language();
+    //kDebug() <<  "Languagelist" << KGlobal::locale()->languageList();
     KGlobal::locale()->insertCatalog("libksane");
     KGlobal::locale()->insertCatalog("sane-backends");
 
-    status = sane_init(&version, &getSaneAuthorization);
-    if (status != SANE_STATUS_GOOD) {
-        kDebug(51004) << "libksane: sane_init() failed("
-                 << sane_strstatus(status) << ")";
-    }
-    else {
-        //kDebug(51004) << "Sane Version = "
-        //         << SANE_VERSION_MAJOR(version) << "."
-        //         << SANE_VERSION_MINORparent(version) << "."
-        //         << SANE_VERSION_BUILD(version);
+    s_objectMutex.lock();
+    s_objectCount++;
+
+    if (s_objectCount == 1) {
+        // only call sane init for the first instance
+        status = sane_init(&version, &KSaneAuth::authorization);
+        if (status != SANE_STATUS_GOOD) {
+            kDebug() << "libksane: sane_init() failed("
+            << sane_strstatus(status) << ")";
+        }
+        else {
+            //kDebug() << "Sane Version = "
+            //         << SANE_VERSION_MAJOR(version) << "."
+            //         << SANE_VERSION_MINORparent(version) << "."
+            //         << SANE_VERSION_BUILD(version);
+        }
     }
+    s_objectMutex.unlock();
+
+    // read the device list to get a list of vendor and model info
+    d->m_findDevThread->start();
 
     d->m_readValsTmr.setSingleShot(true);
-    d->m_startScanTmr.setSingleShot(true);
     connect(&d->m_readValsTmr,   SIGNAL(timeout()), d, SLOT(valReload()));
-    connect(&d->m_startScanTmr,  SIGNAL(timeout()), d, SLOT(startScan()));
 
+    d->m_updProgressTmr.setSingleShot(false);
+    d->m_updProgressTmr.setInterval(300);
+    connect(&d->m_updProgressTmr, SIGNAL(timeout()), d, SLOT(updateProgress()));
+    
     // Forward signals from the private class
     connect(d, SIGNAL(scanProgress(int)), this, SIGNAL(scanProgress(int)));
     connect(d, SIGNAL(imageReady(QByteArray &, int, int, int, int)),
@@ -100,95 +112,101 @@ KSaneWidget::KSaneWidget(QWidget* parent)
     connect(d, SIGNAL(scanDone(int, QString)), this, SIGNAL(scanDone(int, QString)));
     
     // Create the static UI
-    QHBoxLayout *base_layout = new QHBoxLayout;
-    base_layout->setSpacing(2);
-    base_layout->setMargin(0);
-    setLayout(base_layout);
-    QVBoxLayout *options_layout = new QVBoxLayout;
-    options_layout->setSpacing(2);
-    options_layout->setMargin(0);
-    QVBoxLayout *preview_layout = new QVBoxLayout;
-    preview_layout->setSpacing(2);
-    preview_layout->setMargin(0);
-    
-    base_layout->addLayout(options_layout, 0);
-    base_layout->addLayout(preview_layout, 100);
-
     // create the preview
-    d->m_previewViewer = new KSaneViewer(this);
+    d->m_previewViewer = new KSaneViewer(&(d->m_previewImg), this);
     connect(d->m_previewViewer, SIGNAL(newSelection(float, float, float, float)),
             d, SLOT(handleSelection(float, float, float, float)));
 
-    d->m_zInBtn  = new KPushButton(this);
-    d->m_zInBtn->setIcon(KIcon("zoom-in"));
-    d->m_zInBtn->setToolTip(i18n("Zoom In"));
-    d->m_zOutBtn = new KPushButton(this);
-    d->m_zOutBtn->setIcon(KIcon("zoom-out"));
-    d->m_zOutBtn->setToolTip(i18n("Zoom Out"));
-    d->m_zSelBtn = new KPushButton(this);
-    d->m_zSelBtn->setIcon(KIcon("zoom-fit-best"));
-    d->m_zSelBtn->setToolTip(i18n("Zoom to Selection"));
-    d->m_zFitBtn = new KPushButton(this);
-    d->m_zFitBtn->setIcon(KIcon("document-preview"));
-    d->m_zFitBtn->setToolTip(i18n("Zoom to Fit"));
-
-    d->m_warmingUp = new QLabel(this);
-    d->m_warmingUp->setText(i18n("The lamp is warming up."));
+    
+    d->m_warmingUp = new QLabel;
+    d->m_warmingUp->setText(i18n("Waiting for the scan to start."));
     d->m_warmingUp->setAlignment(Qt::AlignCenter);
     d->m_warmingUp->setAutoFillBackground(true);
     d->m_warmingUp->setBackgroundRole(QPalette::Highlight);
     //d->m_warmingUp->setForegroundRole(QPalette::HighlightedText);
     d->m_warmingUp->hide();
 
-    d->m_progressBar = new QProgressBar(this);
-    d->m_progressBar->hide();
+    d->m_progressBar = new QProgressBar;
     d->m_progressBar->setMaximum(100);
 
-    d->m_cancelBtn   = new KPushButton(this);
+    d->m_cancelBtn   = new KPushButton;
     d->m_cancelBtn->setIcon(KIcon("process-stop"));
     d->m_cancelBtn->setToolTip(i18n("Cancel current scan operation"));
-    d->m_cancelBtn->hide();
-
+    connect(d->m_cancelBtn, SIGNAL(clicked()), this, SLOT(scanCancel()));
+    
+    d->m_activityFrame = new QWidget;
+    QHBoxLayout *progress_lay = new QHBoxLayout(d->m_activityFrame);
+    progress_lay->setContentsMargins(0,0,0,0);
+    progress_lay->addWidget(d->m_progressBar, 100);
+    progress_lay->addWidget(d->m_cancelBtn, 0);
+    d->m_activityFrame->hide();
+    
+    d->m_zInBtn  = new QToolButton(this);
+    d->m_zInBtn->setAutoRaise(true);
+    d->m_zInBtn->setIcon(KIcon("zoom-in"));
+    d->m_zInBtn->setToolTip(i18n("Zoom In"));
+    connect(d->m_zInBtn, SIGNAL(clicked()), d->m_previewViewer, SLOT(zoomIn()));
+    
+    d->m_zOutBtn = new QToolButton(this);
+    d->m_zOutBtn->setAutoRaise(true);
+    d->m_zOutBtn->setIcon(KIcon("zoom-out"));
+    d->m_zOutBtn->setToolTip(i18n("Zoom Out"));
+    connect(d->m_zOutBtn, SIGNAL(clicked()), d->m_previewViewer, SLOT(zoomOut()));
+    
+    d->m_zSelBtn = new QToolButton(this);
+    d->m_zSelBtn->setAutoRaise(true);
+    d->m_zSelBtn->setIcon(KIcon("zoom-fit-best"));
+    d->m_zSelBtn->setToolTip(i18n("Zoom to Selection"));
+    connect(d->m_zSelBtn, SIGNAL(clicked()), d->m_previewViewer, SLOT(zoomSel()));
+    
+    d->m_zFitBtn = new QToolButton(this);
+    d->m_zFitBtn->setAutoRaise(true);
+    d->m_zFitBtn->setIcon(KIcon("document-preview"));
+    d->m_zFitBtn->setToolTip(i18n("Zoom to Fit"));
+    connect(d->m_zFitBtn, SIGNAL(clicked()), d->m_previewViewer, SLOT(zoom2Fit()));
+    
     d->m_prevBtn = new KPushButton(this);
     d->m_prevBtn->setIcon(KIcon("document-import"));
     d->m_prevBtn->setToolTip(i18n("Scan Preview Image"));
     d->m_prevBtn->setText(i18nc("Preview button text", "Preview"));
+    connect(d->m_prevBtn,   SIGNAL(clicked()), d, SLOT(startPreviewScan()));
+    
     d->m_scanBtn = new KPushButton(this);
     d->m_scanBtn->setIcon(KIcon("document-save"));
     d->m_scanBtn->setToolTip(i18n("Scan Final Image"));
     d->m_scanBtn->setText(i18nc("Final scan button text", "Scan"));
     d->m_scanBtn->setFocus(Qt::OtherFocusReason);
-
-    connect(d->m_zInBtn,    SIGNAL(clicked()), d->m_previewViewer, SLOT(zoomIn()));
-    connect(d->m_zOutBtn,   SIGNAL(clicked()), d->m_previewViewer, SLOT(zoomOut()));
-    connect(d->m_zSelBtn,   SIGNAL(clicked()), d->m_previewViewer, SLOT(zoomSel()));
-    connect(d->m_zFitBtn,   SIGNAL(clicked()), d->m_previewViewer, SLOT(zoom2Fit()));
-    connect(d->m_scanBtn,   SIGNAL(clicked()), d, SLOT(scanFinal()));
-    connect(d->m_prevBtn,   SIGNAL(clicked()), d, SLOT(scanPreview()));
-    connect(d->m_cancelBtn, SIGNAL(clicked()), this, SLOT(scanCancel()));
-
-    QHBoxLayout *zoom_layout = new QHBoxLayout;
-    QHBoxLayout *progress_lay = new QHBoxLayout;
-
+    connect(d->m_scanBtn,   SIGNAL(clicked()), d, SLOT(startFinalScan()));
+    
+    d->m_btnFrame = new QWidget;
+    QHBoxLayout *btn_lay = new QHBoxLayout(d->m_btnFrame);
+    btn_lay->setContentsMargins(0,0,0,0);
+    btn_lay->addWidget(d->m_zInBtn);
+    btn_lay->addWidget(d->m_zOutBtn);
+    btn_lay->addWidget(d->m_zSelBtn);
+    btn_lay->addWidget(d->m_zFitBtn);
+    btn_lay->addStretch(100);
+    btn_lay->addWidget(d->m_prevBtn);
+    btn_lay->addWidget(d->m_scanBtn);
+    
+    // calculate the height of the waiting/scanning/buttons frames to avoid jumpiness.
+    int minHeight = d->m_btnFrame->sizeHint().height();
+    if (d->m_activityFrame->sizeHint().height() > minHeight) minHeight = d->m_activityFrame->sizeHint().height();
+    if (d->m_warmingUp->sizeHint().height() > minHeight)     minHeight = d->m_warmingUp->sizeHint().height();
+    d->m_btnFrame->setMinimumHeight(minHeight);
+    d->m_activityFrame->setMinimumHeight(minHeight);
+    d->m_warmingUp->setMinimumHeight(minHeight);
+    
+    d->m_previewFrame = new QWidget;
+    QVBoxLayout *preview_layout = new QVBoxLayout(d->m_previewFrame);
+    preview_layout->setContentsMargins(0,0,0,0);
     preview_layout->addWidget(d->m_previewViewer, 100);
-    preview_layout->addLayout(progress_lay, 0);
-    preview_layout->addLayout(zoom_layout, 0);
-
-    progress_lay->addWidget(d->m_warmingUp, 100);
-    progress_lay->addWidget(d->m_progressBar, 100);
-    progress_lay->addWidget(d->m_cancelBtn, 0);
-
-    zoom_layout->addWidget(d->m_zInBtn);
-    zoom_layout->addWidget(d->m_zOutBtn);
-    zoom_layout->addWidget(d->m_zSelBtn);
-    zoom_layout->addWidget(d->m_zFitBtn);
-    zoom_layout->addStretch(100);
-    zoom_layout->addWidget(d->m_prevBtn);
-    zoom_layout->addWidget(d->m_scanBtn);
-
+    preview_layout->addWidget(d->m_warmingUp, 0);
+    preview_layout->addWidget(d->m_activityFrame, 0);
+    preview_layout->addWidget(d->m_btnFrame, 0);
+    
     // Create Options Widget
     d->m_optsTabWidget = new KTabWidget;
-    options_layout->addWidget(d->m_optsTabWidget, 0);
     
     // Add the basic options tab
     d->m_basicScrollA = new QScrollArea;
@@ -200,87 +218,115 @@ KSaneWidget::KSaneWidget(QWidget* parent)
     d->m_otherScrollA = new QScrollArea;
     d->m_otherScrollA->setWidgetResizable(true);
     d->m_otherScrollA->setFrameShape(QFrame::NoFrame);
-    d->m_optsTabWidget->addTab(d->m_otherScrollA, i18n("Other Options"));
+    d->m_optsTabWidget->addTab(d->m_otherScrollA, i18n("Scanner Specific Options"));
+    
+    
+    d->m_splitter = new QSplitter(this);
+    d->m_splitter->addWidget(d->m_optsTabWidget);
+    d->m_splitter->setStretchFactor(0,0);
+    d->m_splitter->addWidget(d->m_previewFrame);
+    d->m_splitter->setStretchFactor(1,100);
+    
+    d->m_optionsCollapser = new SplitterCollapser(d->m_splitter, d->m_optsTabWidget);
+    
+    QHBoxLayout *base_layout = new QHBoxLayout(this);
+    base_layout->addWidget(d->m_splitter);
+    base_layout->setContentsMargins(0,0,0,0);
 
+    // disable the interface in case no device is opened.
+    d->m_optsTabWidget->setDisabled(true);
+    d->m_previewViewer->setDisabled(true);
+    d->m_btnFrame->setDisabled(true);
+    
 }
 
 KSaneWidget::~KSaneWidget()
 {
-    closeDevice();
-    sane_exit();
+    while (!closeDevice()) usleep(1000);
+    // wait for any thread to exit
+
+    s_objectMutex.lock();
+    s_objectCount--;
+    if (s_objectCount <= 0) {
+        // only delete the find-devices and authorization singletons and call sane_exit
+        // if this is the last instance
+        delete d->m_findDevThread;
+        delete d->m_auth;
+        sane_exit();
+    }
+    s_objectMutex.unlock();
     delete d;
 }
 
-QString KSaneWidget::vendor() const {return d->m_vendor;}
-QString KSaneWidget::make() const {return d->m_vendor;}
-QString KSaneWidget::model() const {return d->m_model;}
+QString KSaneWidget::vendor() const 
+{
+    d->m_findDevThread->wait();
+    d->devListUpdated(); // this is just a wrapped if (m_vendor.isEmpty()) statement if the vendor is known
+    // devListUpdated here is to ensure that we do not come in between finished and the devListUpdated slot
+
+    return d->m_vendor;
+}
+QString KSaneWidget::make() const 
+{
+    return vendor();
+}
+QString KSaneWidget::model() const 
+{
+    d->m_findDevThread->wait();
+    d->devListUpdated(); // this is just a wrapped if (m_vendor.isEmpty()) statement if the vendor is known
+    // devListUpdated here is to ensure that we do not come in between finished and the devListUpdated slot
+
+    return d->m_model;
+}
 
 QString KSaneWidget::selectDevice(QWidget* parent)
 {
-  QString selected_name("");
-  KSaneDeviceDialog sel(parent);
+  QString selected_name;
+  QPointer<KSaneDeviceDialog> sel = new KSaneDeviceDialog(parent);
 
-  // sel.setDefault(prev_backend); // set default scanner - perhaps application using libksane should remember that                                                
-  if(sel.exec()) {
-      return selected_name = sel.getSelectedName();
+  // sel.setDefault(prev_backend); // set default scanner - perhaps application using libksane should remember that
+  if(sel->exec() == KDialog::Accepted) {
+      return selected_name = sel->getSelectedName();
   }
+  delete sel;
   return selected_name;
 }
 
-bool KSaneWidget::openDevice(const QString &device_name)
+bool KSaneWidget::openDevice(const QString &deviceName)
 {
     int                            i=0;
     const SANE_Option_Descriptor  *optDesc;
     SANE_Status                    status;
-    SANE_Word                      num_sane_options;
+    SANE_Word                      numSaneOptions;
     SANE_Int                       res;
-    SANE_Device const            **dev_list;
     KPasswordDialog               *dlg;
-    KWallet::Wallet               *sane_wallet;
-    QString                        my_folder_name("ksane");
+    KWallet::Wallet               *saneWallet;
+    QString                        myFolderName("ksane");
     QMap<QString, QString>         wallet_entry;
 
     // don't bother trying to open if the device string is empty
-    if (device_name.isEmpty()) {
+    if (deviceName.isEmpty()) {
         return false;
     }
-
-    // get the device list to get the vendor and model info
-    status = sane_get_devices(&dev_list, SANE_TRUE);
-
-    while(dev_list[i] != 0) {
-        if (QString(dev_list[i]->name) == device_name) {
-            d->m_modelName = QString(dev_list[i]->vendor) + ' ' + QString(dev_list[i]->model);
-            d->m_vendor    = QString(dev_list[i]->vendor);
-            d->m_model     = QString(dev_list[i]->model);
-            break;
-        }
-        i++;
-    }
-
-    if (device_name == "test") {
-        d->m_modelName = "Test Scanner";
-        d->m_vendor    = "Test";
-        d->m_model     = "Scanner";
-    }
-
-    status = sane_open(device_name.toLatin1(), &d->m_saneHandle);
+    // save the device name
+    d->m_devName = deviceName;
 
     // Try to open the device
+    status = sane_open(deviceName.toLatin1(), &d->m_saneHandle);
 
     bool password_dialog_ok = true;
 
     // prepare wallet for authentication and create password dialog
     if(status == SANE_STATUS_ACCESS_DENIED) {
-        sane_wallet = KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet(), winId() );
+        saneWallet = KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet(), winId() );
 
-        if(sane_wallet) {
+        if(saneWallet) {
             dlg = new KPasswordDialog(this, KPasswordDialog::ShowUsernameLine | KPasswordDialog::ShowKeepPassword);
-            if(!sane_wallet->hasFolder(my_folder_name)) {
-                sane_wallet->createFolder(my_folder_name);
+            if(!saneWallet->hasFolder(myFolderName)) {
+                saneWallet->createFolder(myFolderName);
             }
-            sane_wallet->setFolder(my_folder_name);
-            sane_wallet->readMap(device_name.toLatin1(), wallet_entry);
+            saneWallet->setFolder(myFolderName);
+            saneWallet->readMap(deviceName.toLatin1(), wallet_entry);
             if(!wallet_entry.empty() || true) {
                 dlg->setUsername( wallet_entry["username"] );
                 dlg->setPassword( wallet_entry["password"] );
@@ -289,7 +335,7 @@ bool KSaneWidget::openDevice(const QString &device_name)
         } else {
             dlg = new KPasswordDialog(this, KPasswordDialog::ShowUsernameLine);
         }
-        dlg->setPrompt(i18n("Authentication required for resource: %1", device_name ) );
+        dlg->setPrompt(i18n("Authentication required for resource: %1", deviceName ) );
 
     }
 
@@ -300,46 +346,62 @@ bool KSaneWidget::openDevice(const QString &device_name)
         password_dialog_ok = dlg->exec();
         if(!password_dialog_ok) {
             delete dlg;
+            d->m_devName.clear();
             return false; //the user canceled
         }
 
-        sane_username = dlg->username();
-        sane_password = dlg->password();
+        // add/update the device user-name and password for authentication
+        d->m_auth->setDeviceAuth(d->m_devName, dlg->username(), dlg->password());
 
-        status = sane_open(device_name.toLatin1(), &d->m_saneHandle);
+        status = sane_open(deviceName.toLatin1(), &d->m_saneHandle);
 
-        // store password in wallet on sucessful auth
+        // store password in wallet on successful authentication
         if(dlg->keepPassword() && status != SANE_STATUS_ACCESS_DENIED) {
             QMap<QString, QString> entry;
             entry["username"] = dlg->username().toUtf8();
             entry["password"] = dlg->password().toUtf8();
-            sane_wallet->writeMap(device_name.toLatin1(), entry);
+            saneWallet->writeMap(deviceName.toLatin1(), entry);
         }
     }
 
-    // clear static members in library
-    sane_username = "";
-    sane_password = "";
-
     if (status != SANE_STATUS_GOOD) {
-        qDebug() << "sane_open(\"" << device_name << "\", &handle) failed! status = " << status;
+        kDebug() << "sane_open(\"" << deviceName << "\", &handle) failed! status = " << sane_strstatus(status);
+        d->m_auth->clearDeviceAuth(d->m_devName);
+        d->m_devName.clear();
         return false;
     }
 
+    // update the device list if needed to get the vendor and model info
+    if (d->m_findDevThread->devicesList().size() == 0) {
+        d->m_findDevThread->start();
+    }
+    else {
+        // us the "old" existing list
+        d->devListUpdated();
+        // if m_vendor is not updated it means that the list needs to be updated.
+        if (d->m_vendor.isEmpty()) {
+            d->m_findDevThread->start();
+        }
+    }
+
     // Read the options (start with option 0 the number of parameters)
     optDesc = sane_get_option_descriptor(d->m_saneHandle, 0);
     if (optDesc == 0) {
+        d->m_auth->clearDeviceAuth(d->m_devName);
+        d->m_devName.clear();
         return false;
     }
     QVarLengthArray<char> data(optDesc->size);
     status = sane_control_option(d->m_saneHandle, 0, SANE_ACTION_GET_VALUE, data.data(), &res);
     if (status != SANE_STATUS_GOOD) {
+        d->m_auth->clearDeviceAuth(d->m_devName);
+        d->m_devName.clear();
         return false;
     }
-    num_sane_options = *reinterpret_cast<SANE_Word*>(data.data());
+    numSaneOptions = *reinterpret_cast<SANE_Word*>(data.data());
 
     // read the rest of the options
-    for (i=1; i<num_sane_options; i++) {
+    for (i=1; i<numSaneOptions; ++i) {
         switch (KSaneOption::otpionType(sane_get_option_descriptor(d->m_saneHandle, i))) {
             case KSaneOption::TYPE_DETECT_FAIL:
                 d->m_optList.append(new KSaneOption(d->m_saneHandle, i));
@@ -369,45 +431,79 @@ bool KSaneWidget::openDevice(const QString &device_name)
     }
 
     // do the connections of the option parameters
-    for (i=1; i<d->m_optList.size(); i++) {
+    for (i=1; i<d->m_optList.size(); ++i) {
+        //kDebug() << d->m_optList.at(i)->name();
         connect (d->m_optList.at(i), SIGNAL(optsNeedReload()), d, SLOT(optReload()));
         connect (d->m_optList.at(i), SIGNAL(valsNeedReload()), d, SLOT(scheduleValReload()));
     }
-
+    
+    // Create the preview thread
+    d->m_previewThread = new KSanePreviewThread(d->m_saneHandle, &d->m_previewImg);
+    connect(d->m_previewThread, SIGNAL(finished()), d, SLOT(previewScanDone()));
+    
     // Create the read thread
-    d->m_readThread = new KSaneReadThread(d->m_saneHandle, d->m_saneReadBuffer, IMG_DATA_R_SIZE);
-    connect(d->m_readThread, SIGNAL(finished()), d, SLOT(processData()));
-
+    d->m_scanThread = new KSaneScanThread(d->m_saneHandle, &d->m_scanData);
+    connect(d->m_scanThread, SIGNAL(finished()), d, SLOT(oneFinalScanDone()));
+    
     // Create the options interface
     d->createOptInterface();
-
+    
     // try to set KSaneWidget default values
     d->setDefaultValues();
+    
+    // Enable the interface
+    d->m_optsTabWidget->setDisabled(false);
+    d->m_previewViewer->setDisabled(false);
+    d->m_btnFrame->setDisabled(false);
 
     // estimate the preview size and create an empty image
-    // this is done so that you can select scanarea without
+    // this is done so that you can select scan area without
     // having to scan a preview.
     d->updatePreviewSize();
-
+    QTimer::singleShot(1000, d->m_previewViewer, SLOT(zoom2Fit()));
     return true;
 }
 
 
 bool KSaneWidget::closeDevice()
 {
-    if (d->m_saneHandle) {
-        d->scanCancel();
-        sane_close(d->m_saneHandle);
+    if (!d->m_saneHandle) {
+        return true;
     }
+    
+    if (d->m_scanThread->isRunning()) {
+        d->m_scanThread->cancelScan();
+        d->m_closeDevicePending = true;
+        return false;
+    }
+    
+    if (d->m_previewThread->isRunning()) {
+        d->m_previewThread->cancelScan();
+        d->m_closeDevicePending = true;
+        return false;
+    }
+
+    d->m_auth->clearDeviceAuth(d->m_devName);
+    // else 
+    sane_close(d->m_saneHandle);
+    d->m_saneHandle = 0;
     d->clearDeviceOptions();
+
+    // disable the interface until a new device is opened.
+    d->m_optsTabWidget->setDisabled(true);
+    d->m_previewViewer->setDisabled(true);
+    d->m_btnFrame->setDisabled(true);
+    
     return true;
 }
 
-QImage KSaneWidget::toQImage(const QByteArray &data,
-                              int width,
-                              int height,
-                              int bytes_per_line,
-                              ImageFormat format)
+#define inc_pixel(x,y,ppl) { x++; if (x>=ppl) { y++; x=0;} }
+
+QImage KSaneWidget::toQImageSilent(const QByteArray &data,
+                                   int width,
+                                   int height,
+                                   int bytes_per_line,
+                                   ImageFormat format)
 {
     QImage img;
     int j=0;
@@ -453,8 +549,6 @@ QImage KSaneWidget::toQImage(const QByteArray &data,
                 img.bits()[j+2] = data.data()[i];
                 j+=4;
             }
-            KMessageBox::sorry(0, i18n("The image data contained 16 bits per color, "
-                    "but the color depth has been truncated to 8 bits per color."));
             break;
 
         case FormatRGB_8_C:
@@ -493,13 +587,11 @@ QImage KSaneWidget::toQImage(const QByteArray &data,
 
                 inc_pixel(pixel_x, pixel_y, width);
             }
-            KMessageBox::sorry(0, i18n("The image data contained 16 bits per color, "
-                    "but the color depth has been truncated to 8 bits per color."));
             break;
 
         case FormatNone:
         default:
-            kDebug(51004) << "Unsupported conversion";
+            kDebug() << "Unsupported conversion";
             break;
     }
     float dpm = currentDPI() * (1000.0 / 25.4);
@@ -508,15 +600,40 @@ QImage KSaneWidget::toQImage(const QByteArray &data,
     return img;
 }
 
+QImage KSaneWidget::toQImage(const QByteArray &data,
+                              int width,
+                              int height,
+                              int bytes_per_line,
+                              ImageFormat format)
+{
+    
+    if ((format == FormatRGB_16_C) || (format == FormatGrayScale16)) {
+            KMessageBox::sorry(0, i18n("The image data contained 16 bits per color, "
+                    "but the color depth has been truncated to 8 bits per color."));
+    }
+    return toQImageSilent(data, width, height, bytes_per_line, format);
+}
+
 void KSaneWidget::scanFinal()
 {
-    d->scanFinal();
+    if (d->m_btnFrame->isEnabled()) {
+        d->startFinalScan();
+    }
+    else {
+        // if the button frame is disabled, there is no open device to scan from
+        emit scanDone(KSaneWidget::ErrorGeneral, "");
+    }
 }
 
 void KSaneWidget::scanCancel()
 {
-    d->scanCancel();
-    emit scanProgress(0);
+    if (d->m_scanThread->isRunning()) {
+        d->m_scanThread->cancelScan();
+    }
+    
+    if (d->m_previewThread->isRunning()) {
+        d->m_previewThread->cancelScan();
+    }
 }
 
 void KSaneWidget::getOptVals(QMap <QString, QString> &opts)
@@ -533,6 +650,11 @@ void KSaneWidget::getOptVals(QMap <QString, QString> &opts)
     }
 }
 
+void KSaneWidget::setPreviewResolution(float dpi)
+{
+    d->m_previewDPI = dpi;
+}
+
 bool KSaneWidget::getOptVal(const QString &optname, QString &value)
 {
     KSaneOption *option;
@@ -655,4 +777,99 @@ float KSaneWidget::currentDPI()
     return 0.0;
 }
 
+float KSaneWidget::scanAreaWidth()
+{
+    float result = 0.0;
+    if (d->m_optBrX) {
+        if (d->m_optBrX->getUnit() == SANE_UNIT_PIXEL) {
+            d->m_optBrX->getMaxValue(result);
+            result = result / currentDPI() / 25.4;
+        }
+        else if (d->m_optBrX->getUnit() == SANE_UNIT_MM) {
+            d->m_optBrX->getMaxValue(result);
+        }
+    }
+    return result;
+}
+
+float KSaneWidget::scanAreaHeight()
+{
+    float result = 0.0;
+    if (d->m_optBrY) {
+        if (d->m_optBrY->getUnit() == SANE_UNIT_PIXEL) {
+            d->m_optBrY->getMaxValue(result);
+            result = result / currentDPI() / 25.4;
+        }
+        else if (d->m_optBrY->getUnit() == SANE_UNIT_MM) {
+            d->m_optBrY->getMaxValue(result);
+        }
+    }
+    return result;
+}
+
+void KSaneWidget::setSelection(QPointF topLeft, QPointF bottomRight)
+{
+    if (!d->m_optBrX || !d->m_optBrY || !d->m_optTlX || !d->m_optTlY) {
+        return;
+    }
+    float xmax, ymax;
+    d->m_optBrX->getMaxValue(xmax);
+    d->m_optBrY->getMaxValue(ymax);
+    if (topLeft.x() < 0.0 || topLeft.y() < 0.0 || bottomRight.x() < 0.0 || bottomRight.y() < 0.0) {
+        d->m_previewViewer->clearActiveSelection();
+        d->m_optTlX->setValue(0.0);
+        d->m_optTlY->setValue(0.0);
+        d->m_optBrX->setValue(xmax);
+        d->m_optBrY->setValue(ymax);
+    }
+
+    if (d->m_optBrY->getUnit() == SANE_UNIT_MM) {
+        // clear selection if values are out of bounds
+        if (topLeft.x() > xmax || topLeft.y() > ymax || bottomRight.x() > xmax || bottomRight.y() > ymax) {
+            d->m_previewViewer->clearActiveSelection();
+            d->m_optTlX->setValue(0.0);
+            d->m_optTlY->setValue(0.0);
+            d->m_optBrX->setValue(xmax);
+            d->m_optBrY->setValue(ymax);
+        }
+        d->m_previewViewer->setSelection(topLeft.x(), topLeft.y(), bottomRight.x(), bottomRight.y());
+        d->m_optTlX->setValue(topLeft.x());
+        d->m_optTlY->setValue(topLeft.y());
+        d->m_optBrX->setValue(bottomRight.x());
+        d->m_optBrY->setValue(bottomRight.y());
+    }
+    else if (d->m_optBrY->getUnit() == SANE_UNIT_PIXEL) {
+        const float mmperinch = 25.4;
+        const float dpi = currentDPI();
+        const float m = dpi / mmperinch;
+        if (m*topLeft.x() > xmax || m*topLeft.y() > ymax || m*bottomRight.x() > xmax || m*bottomRight.y() > ymax) {
+            d->m_previewViewer->clearActiveSelection();
+            d->m_optTlX->setValue(0.0);
+            d->m_optTlY->setValue(0.0);
+            d->m_optBrX->setValue(xmax);
+            d->m_optBrY->setValue(ymax);
+        }
+        d->m_previewViewer->setSelection(m*topLeft.x(), m*topLeft.y(), m*bottomRight.x(), m*bottomRight.y());
+        d->m_optTlX->setValue(m*topLeft.x());
+        d->m_optTlY->setValue(m*topLeft.y());
+        d->m_optBrX->setValue(m*bottomRight.x());
+        d->m_optBrY->setValue(m*bottomRight.y());
+    }
+}
+
+void KSaneWidget::setOptionsCollapsed(bool collapse)
+{
+    if (collapse) {
+        QTimer::singleShot(0, d->m_optionsCollapser, SLOT(slotCollapse()));
+    }
+    else {
+        QTimer::singleShot(0, d->m_optionsCollapser, SLOT(slotRestore()));
+    }
+}
+
+void KSaneWidget::setScanButtonHidden(bool hidden)
+{
+    d->m_scanBtn->setHidden(hidden);
+}
+
 }  // NameSpace KSaneIface
