@@ -34,9 +34,6 @@
 #include <QSlider>
 #include <QGridLayout>
 
-/** Remove #if tags when KDE 4.2 becomes unpopular**/
-#include <kdeversion.h>
-
 /**
   *@author Kåre Särs
   */
@@ -89,7 +86,6 @@ public Q_SLOTS:
     void setStep(int);
     /** Set the unit */
     void setSuffix(const KLocalizedString &text);
-    void setSuffix1(const QString &text); /** Remove this compatibility hack when KDE 4.2 becomes unpopular **/
 
 private Q_SLOTS:
 
