@@ -340,6 +340,25 @@ class OKULAR_EXPORT Generator : public QObject
          */
         virtual bool print( QPrinter &printer );
 
+        /**
+         * Possible print errors
+         * @since 0.11 (KDE 4.5)
+         */
+        enum PrintError
+        {
+            NoPrintError,                       ///< There was no print error
+            UnknownPrintError,
+            TemporaryFileOpenPrintError,
+            FileConversionPrintError,
+            PrintingProcessCrashPrintError,
+            PrintingProcessStartPrintError,
+            PrintToFilePrintError,
+            InvalidPrinterStatePrintError,
+            UnableToFindFilePrintError,
+            NoFileToPrintError,
+            NoBinaryToPrintError
+        };
+        
         /**
          * This method returns the meta data of the given @p key with the given @p option
          * of the document.
@@ -470,6 +489,12 @@ class OKULAR_EXPORT Generator : public QObject
         * @since 0.10 (KDE 4.4)
         */
         const SourceReference * dynamicSourceReference( int pageNr, double absX, double absY );
+        
+        /**
+         * Returns the last print error in case print() failed
+         * @since 0.11 (KDE 4.5)
+         */
+        Okular::Generator::PrintError printError() const;
 
     protected:
         /// @cond PRIVATE
@@ -563,8 +588,12 @@ class OKULAR_EXPORT PixmapRequest
 
 }
 
+Q_DECLARE_METATYPE(Okular::Generator::PrintError)
+
 #ifndef QT_NO_DEBUG_STREAM
 OKULAR_EXPORT QDebug operator<<( QDebug str, const Okular::PixmapRequest &req );
 #endif
 
 #endif
+
+/* kate: replace-tabs on; indent-width 4; */
