@@ -275,7 +275,7 @@ static KAboutData createAboutData()
          "okular_poppler",
          "okular_poppler",
          ki18n( "PDF Backend" ),
-         "0.2.2",
+         "0.4",
          ki18n( "A PDF file renderer" ),
          KAboutData::License_GPL,
          ki18n( "© 2005-2008 Albert Astals Cid" )
@@ -290,7 +290,8 @@ PDFGenerator::PDFGenerator( QObject *parent, const QVariantList &args )
     : Generator( parent, args ), pdfdoc( 0 ), ready( true ),
     pixmapRequest( 0 ), docInfoDirty( true ), docSynopsisDirty( true ),
     docEmbeddedFilesDirty( true ), nextFontPage( 0 ),
-    dpiX( 72.0 /*Okular::Utils::dpiX()*/ ), dpiY( 72.0 /*Okular::Utils::dpiY()*/ )
+    dpiX( 72.0 /*Okular::Utils::dpiX()*/ ), dpiY( 72.0 /*Okular::Utils::dpiY()*/ ),
+    synctex_scanner(0)
 {
     setFeature( TextExtraction );
     setFeature( FontInfo );
@@ -328,9 +329,15 @@ bool PDFGenerator::loadDocument( const QString & filePath, QVector<Okular::Page*
     // create PDFDoc for the given file
     pdfdoc = Poppler::Document::load( filePath, 0, 0 );
     bool success = init(pagesVector, filePath.section('/', -1, -1));
-    if (success && QFile::exists(filePath + QLatin1String( "sync" )))
+    if (success)
     {
-        loadPdfSync(filePath, pagesVector);
+        // no need to check for the existence of a synctex file, no parser will be
+        // created if none exists
+        initSynctexParser(filePath);
+        if ( !synctex_scanner && QFile::exists(filePath + QLatin1String( "sync" ) ) )
+        {
+            loadPdfSync(filePath, pagesVector);
+        }
     }
     return success;
 }
@@ -388,7 +395,7 @@ bool PDFGenerator::init(QVector<Okular::Page*> & pagesVector, const QString &wal
         {
             QString prompt;
             if ( firstInput )
-                prompt = i18n( "Please insert the password to read the document:" );
+                prompt = i18n( "Please enter the password to read the document:" );
             else
                 prompt = i18n( "Incorrect password. Try again:" );
             firstInput = false;
@@ -449,6 +456,11 @@ bool PDFGenerator::doCloseDocument()
     docEmbeddedFiles.clear();
     nextFontPage = 0;
     rectsGenerated.clear();
+    if ( synctex_scanner )
+    {
+        synctex_scanner_free( synctex_scanner );
+        synctex_scanner = 0;
+    }
 
     return true;
 }
@@ -874,11 +886,15 @@ bool PDFGenerator::print( QPrinter& printer )
     KTemporaryFile tf;
     tf.setSuffix( ".ps" );
     if ( !tf.open() )
+    {
+        lastPrintError = TemporaryFileOpenPrintError;
         return false;
+    }
     QString tempfilename = tf.fileName();
 
     // Generate the list of pages to be printed as selected in the print dialog
     QList<int> pageList = Okular::FilePrinter::pageList( printer, pdfdoc->numPages(),
+                                                         document()->currentPage() + 1,
                                                          document()->bookmarkedPageList() );
 
     // TODO rotation
@@ -922,13 +938,16 @@ bool PDFGenerator::print( QPrinter& printer )
                                                   Okular::FilePrinter::SystemDeletesFiles,
                                                   Okular::FilePrinter::ApplicationSelectsPages,
                                                   document()->bookmarkedPageRange() );
-        if ( ret >= 0 ) return true;
+
+        lastPrintError = Okular::FilePrinter::printError( ret );
+
+        return (lastPrintError == NoPrintError);
     }
     else
     {
+        lastPrintError = FileConversionPrintError;
         delete psConverter;
         userMutex()->unlock();
-        return false;
     }
 
     tf.close();
@@ -940,23 +959,35 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
 {
     if ( key == "StartFullScreen" )
     {
+        QMutexLocker ml(userMutex());
         // asking for the 'start in fullscreen mode' (pdf property)
         if ( pdfdoc->pageMode() == Poppler::Document::FullScreen )
             return true;
     }
     else if ( key == "NamedViewport" && !option.toString().isEmpty() )
     {
-        // asking for the page related to a 'named link destination'. the
-        // option is the link name. @see addSynopsisChildren.
         Okular::DocumentViewport viewport;
-        userMutex()->lock();
-        Poppler::LinkDestination *ld = pdfdoc->linkDestination( option.toString() );
-        userMutex()->unlock();
-        if ( ld )
+        QString optionString = option.toString();
+
+        // if option starts with "src:" assume that we are handling a 
+        // source reference
+        if ( optionString.startsWith( "src:", Qt::CaseInsensitive ) )
         {
-            fillViewportFromLinkDestination( viewport, *ld );
+            fillViewportFromSourceReference( viewport, optionString );
+        }
+        else
+        {
+            // asking for the page related to a 'named link destination'. the
+            // option is the link name. @see addSynopsisChildren.
+            userMutex()->lock();
+            Poppler::LinkDestination *ld = pdfdoc->linkDestination( optionString );
+            userMutex()->unlock();
+            if ( ld )
+            {
+                fillViewportFromLinkDestination( viewport, *ld );
+            }
+            delete ld;
         }
-        delete ld;
         if ( viewport.pageNumber >= 0 )
             return viewport.toString();
     }
@@ -969,12 +1000,14 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
     }
     else if ( key == "OpenTOC" )
     {
+        QMutexLocker ml(userMutex());
         if ( pdfdoc->pageMode() == Poppler::Document::UseOutlines )
             return true;
     }
 #ifdef HAVE_POPPLER_0_9
     else if ( key == "DocumentScripts" && option.toString() == "JavaScript" )
     {
+        QMutexLocker ml(userMutex());
         return pdfdoc->scripts();
     }
 #endif
@@ -999,7 +1032,7 @@ bool PDFGenerator::reparseConfig()
         userMutex()->unlock();
         somethingchanged = true;
     }
-    bool aaChanged = setAAOptions();
+    bool aaChanged = setDocumentRenderHints();
     somethingchanged = somethingchanged || aaChanged;
     return somethingchanged;
 }
@@ -1008,7 +1041,7 @@ void PDFGenerator::addPages( KConfigDialog * )
 {
 }
 
-bool PDFGenerator::setAAOptions()
+bool PDFGenerator::setDocumentRenderHints()
 {
     bool changed = false;
     static Poppler::Document::RenderHints oldhints = 0;
@@ -1027,6 +1060,9 @@ bool PDFGenerator::setAAOptions()
 }
     SET_HINT("GraphicsAntialias", true, Poppler::Document::Antialiasing)
     SET_HINT("TextAntialias", true, Poppler::Document::TextAntialiasing)
+#ifdef HAVE_POPPLER_0_12_1
+    SET_HINT("TextHinting", false, Poppler::Document::TextHinting)
+#endif
 #undef SET_HINT
     return changed;
 }
@@ -1345,12 +1381,8 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
     QHash<int, pdfsyncpoint> points;
     QStack<QString> fileStack;
     int currentpage = -1;
-    QRegExp newfilere( "\\(\\s*([^\\s]+)" );
-    QRegExp linere( "l\\s+(\\d+)\\s+(\\d+)(\\s+(\\d+))?" );
-    QRegExp pagere( "s\\s+(\\d+)" );
-    QRegExp locre( "p\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
-    QRegExp locstarre( "p\\*\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
     const QLatin1String texStr( ".tex" );
+    const QChar spaceChar = QChar::fromLatin1( ' ' );
 
     fileStack.push( coreName + texStr );
 
@@ -1358,45 +1390,51 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
     while ( !ts.atEnd() )
     {
         line = ts.readLine();
-        if ( line.startsWith( QLatin1Char( 'l' ) ) && linere.exactMatch( line ) )
+        const QStringList tokens = line.split( spaceChar, QString::SkipEmptyParts );
+        const int tokenSize = tokens.count();
+        if ( tokenSize < 1 )
+            continue;
+        if ( tokens.first() == QLatin1String( "l" ) && tokenSize >= 3 )
         {
-            int id = linere.cap( 1 ).toInt();
+            int id = tokens.at( 1 ).toInt();
             QHash<int, pdfsyncpoint>::const_iterator it = points.constFind( id );
             if ( it == points.constEnd() )
             {
                 pdfsyncpoint pt;
                 pt.x = 0;
                 pt.y = 0;
-                pt.row = linere.cap( 2 ).toInt();
+                pt.row = tokens.at( 2 ).toInt();
                 pt.column = 0; // TODO
                 pt.page = -1;
                 pt.file = fileStack.top();
                 points[ id ] = pt;
             }
         }
-        else if ( line.startsWith( QLatin1Char( 's' ) ) && pagere.exactMatch( line ) )
+        else if ( tokens.first() == QLatin1String( "s" ) && tokenSize >= 2 )
         {
-            currentpage = pagere.cap( 1 ).toInt() - 1;
+            currentpage = tokens.at( 1 ).toInt() - 1;
         }
-        else if ( line.startsWith( QLatin1String( "p*" ) ) && locstarre.exactMatch( line ) )
+        else if ( tokens.first() == QLatin1String( "p*" ) && tokenSize >= 4 )
         {
             // TODO
             kDebug(PDFDebug) << "PdfSync: 'p*' line ignored";
         }
-        else if ( line.startsWith( QLatin1Char( 'p' ) ) && locre.exactMatch( line ) )
+        else if ( tokens.first() == QLatin1String( "p" ) && tokenSize >= 4 )
         {
-            int id = locre.cap( 1 ).toInt();
+            int id = tokens.at( 1 ).toInt();
             QHash<int, pdfsyncpoint>::iterator it = points.find( id );
             if ( it != points.end() )
             {
-                it->x = locre.cap( 2 ).toInt();
-                it->y = locre.cap( 3 ).toInt();
+                it->x = tokens.at( 2 ).toInt();
+                it->y = tokens.at( 3 ).toInt();
                 it->page = currentpage;
             }
         }
-        else if ( line.startsWith( QLatin1Char( '(' ) ) && newfilere.exactMatch( line ) )
+        else if ( line.startsWith( QLatin1Char( '(' ) ) && tokenSize == 1 )
         {
-            QString newfile = newfilere.cap( 1 );
+            QString newfile = line;
+            // chop the leading '('
+            newfile.remove( 0, 1 );
             if ( !newfile.endsWith( texStr ) )
             {
                 newfile += texStr;
@@ -1424,7 +1462,7 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
         if ( pt.page < 0 || pt.page >= pagesVector.size() )
             continue;
 
-        // maginc numbers for TeX's RSU's (Ridiculously Small Units) conversion to pixels
+        // magic numbers for TeX's RSU's (Ridiculously Small Units) conversion to pixels
         Okular::NormalizedPoint p(
             ( pt.x * dpiX ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->width() ),
             ( pt.y * dpiY ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->height() )
@@ -1438,6 +1476,99 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
             pagesVector[i]->setSourceReferences( refRects.at(i) );
 }
 
+void PDFGenerator::initSynctexParser( const QString& filePath )
+{
+    synctex_scanner = synctex_scanner_new_with_output_file( QFile::encodeName( filePath ), 0, 1);
+}
+
+const Okular::SourceReference * PDFGenerator::dynamicSourceReference( int pageNr, double absX, double absY )
+{
+    if  ( !synctex_scanner )
+        return 0;
+
+    if (synctex_edit_query(synctex_scanner, pageNr + 1, absX * 72. / dpiX, absY * 72. / dpiY) > 0)
+    {
+        synctex_node_t node;
+        // TODO what should we do if there is really more than one node?
+        while (( node = synctex_next_result( synctex_scanner ) ))
+        {
+            int line = synctex_node_line(node);
+            int col = synctex_node_column(node);
+            // column extraction does not seem to be implemented in synctex so far. set the SourceReference default value.
+            if ( col == -1 )
+            {
+                col = 0;
+            }
+            const char *name = synctex_scanner_get_name( synctex_scanner, synctex_node_tag( node ) );
+
+            Okular::SourceReference * sourceRef = new Okular::SourceReference( name, line, col );
+            return sourceRef;
+        }
+    }
+    return 0;
+}
+
+PDFGenerator::PrintError PDFGenerator::printError() const
+{
+    return lastPrintError;
+}
+
+void PDFGenerator::fillViewportFromSourceReference( Okular::DocumentViewport & viewport, const QString & reference ) const
+{
+    if ( !synctex_scanner )
+        return;
+
+    // The reference is of form "src:1111Filename", where "1111"
+    // points to line number 1111 in the file "Filename".
+    // Extract the file name and the numeral part from the reference string.
+    // This will fail if Filename starts with a digit.
+    QString name, lineString;
+    // Remove "src:". Presence of substring has been checked before this 
+    // function is called.
+    name = reference.mid( 4 );
+    // split
+    int nameLength = name.length();
+    int i = 0;
+    for( i = 0; i < nameLength; ++i )
+    {
+        if ( !name[i].isDigit() ) break;
+    }
+    lineString = name.left( i );
+    name = name.mid( i );
+    // Remove spaces.
+    name = name.trimmed();
+    lineString = lineString.trimmed();
+    // Convert line to integer. 
+    bool ok;
+    int line = lineString.toInt( &ok );
+    if (!ok) line = -1;
+    
+    // Use column == -1 for now.
+    if( synctex_display_query( synctex_scanner, name.toLatin1(), line, -1 ) > 0 )
+    {
+        synctex_node_t node;
+        // For now use the first hit. Could possibly be made smarter
+        // in case there are multiple hits.
+        while( ( node = synctex_next_result( synctex_scanner ) ) ) 
+        {
+            // TeX pages start at 1.
+            viewport.pageNumber = synctex_node_page( node ) - 1;
+
+            if ( !viewport.isValid() ) return;
+
+            // TeX small points ...
+            double px = (synctex_node_visible_h( node ) * dpiX) / 72.27;
+            double py = (synctex_node_visible_v( node ) * dpiY) / 72.27;
+            viewport.rePos.normalizedX = px / document()->page(viewport.pageNumber)->width();
+            viewport.rePos.normalizedY = ( py + 0.5 ) / document()->page(viewport.pageNumber)->height();
+            viewport.rePos.enabled = true;
+            viewport.rePos.pos = Okular::DocumentViewport::Center;
+           
+            return;
+        }
+    }
+}
+
 QWidget* PDFGenerator::printConfigurationWidget() const
 {
     if ( !pdfOptionsPage )
@@ -1458,7 +1589,7 @@ bool PDFGenerator::supportsOption( SaveOption option ) const
     return false;
 }
 
-bool PDFGenerator::save( const QString &fileName, SaveOptions options )
+bool PDFGenerator::save( const QString &fileName, SaveOptions options, QString *errorText )
 {
     Poppler::PDFConverter *pdfConv = pdfdoc->pdfConverter();
 
@@ -1468,6 +1599,26 @@ bool PDFGenerator::save( const QString &fileName, SaveOptions options )
 
     QMutexLocker locker( userMutex() );
     bool success = pdfConv->convert();
+#ifdef HAVE_POPPLER_0_12_1
+    if (!success)
+    {
+        switch (pdfConv->lastError())
+        {
+            case Poppler::BaseConverter::NotSupportedInputFileError:
+                *errorText = i18n("Saving files with /Encrypt is not supported.");
+            break;
+
+            case Poppler::BaseConverter::NoError:
+            case Poppler::BaseConverter::FileLockedError:
+                // we can't get here
+            break;
+
+            case Poppler::BaseConverter::OpenOutputError:
+                // the default text message is good for this case
+            break;
+        }
+    }
+#endif
     delete pdfConv;
     return success;
 }
